# -*- encoding: utf-8
import uuid

from django.db import models
from django.conf import settings
from django.contrib.auth.models import User


class SoundFile(models.Model):
    TYPE_CHOICES = (
        ('wav', 'wav'),
    )
    uuid = models.UUIDField(primary_key=True, default=uuid.uuid4)
    file = models.FileField(upload_to=settings.OUTPUT_DIR)
    text = models.TextField(blank=False)
    command = models.CharField(max_length=255, blank=True)
    type = models.CharField(max_length=5, choices=TYPE_CHOICES)
    dc = models.DateTimeField(auto_now_add=True)
    user = models.ForeignKey(User, blank=True, null=True)

    def __str__(self):
        return '%s' % self.uuid


# Create your views here.
import subprocess
import os
import uuid

from django.http import JsonResponse
from django.http import HttpResponse
from django.core.files import File
from rest_framework.response import Response
from rest_framework import status
from rest_framework.decorators import api_view

from . import models
from . import serializers

@api_view(['post'])
def generate(request):
    """
    ---
    parameters:
    - name: text
      description: Text for generation
      required: true
      paramType: form
    """
    serializer = serializers.Generation(data=request.data)
    serializer.is_valid(raise_exception=True)
    text = serializer.validated_data['text']
    tmp_dir = '/tmp/voice/'
    if not os.path.exists(tmp_dir):
        os.makedirs(tmp_dir)

    file_uuid = str(uuid.uuid4())
    tmp_path = '%s/%s.wav' % (tmp_dir, file_uuid)

    command = 'echo %s | RHVoice-test -p elena -o %s' % (text, tmp_path)
    status_code, output = subprocess.getstatusoutput(command)
    if status_code != 0:
        return Response(
            {'error': 'File generation error', 'output': output},
            status=status.HTTP_500_INTERNAL_SERVER_ERROR
        )
    with open(tmp_path, 'rb') as tmp_file:
        models.SoundFile.objects.create(
            uuid=file_uuid,
            text=text,
            command=command,
            file=File(tmp_file),
            type='wav',
        )

    os.unlink(tmp_path)
    return JsonResponse({'status': status_code, 'uuid': file_uuid})


@api_view()
def get_file(request):
    """
    ---
    parameters:
    - name: uuid
      description: uuid of generated file
      required: true
      paramType: query
    """
    serializer = serializers.GetFile(data=request.query_params)
    serializer.is_valid(raise_exception=True)

    sound_file = models.SoundFile.objects.get(uuid=serializer.validated_data['uuid'])

    response = HttpResponse()
    response.write(sound_file.file.read())

    response['Content-Type'] = 'audio/vnd.wave'
    response['Content-Length'] = sound_file.file.size
    response['Content-Disposition'] = 'attachment; filename=%s' % sound_file.file.name.split('/')[-1]

    return response



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
from django.conf import settings
import uuid
import os


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='SoundFile',
            fields=[
                ('uuid', models.UUIDField(default=uuid.uuid4, serialize=False, primary_key=True)),
                ('file', models.FileField(upload_to=os.path.join(settings.BASE_DIR, 'generated'))),
                ('text', models.TextField()),
                ('command', models.CharField(max_length=255, blank=True)),
                ('type', models.CharField(max_length=5, choices=[(b'wav', b'wav')])),
                ('dc', models.DateTimeField(auto_now_add=True)),
                ('user', models.ForeignKey(blank=True, to=settings.AUTH_USER_MODEL, null=True)),
            ],
        ),
    ]



# coding: utf-8
import time
import datetime
from django.core.management.base import BaseCommand
from django.utils.timezone import now
from django.conf import settings
from . import models


class Command(BaseCommand):
    help = 'Clean sound files.'

    def add_arguments(self, parser):
        parser.add_argument('-i', '--infinitely', dest='infinitely', action='store_true')

    def handle(self, *args, **options):
        infinitely = options.get('infinitely')

        while True:
            dead_line = now() - datetime.timedelta(seconds=settings.MAX_SOUND_LIFE)
            models.SoundFile.objects.filter(dc__lte=dead_line).delete()

            if infinitely:
                time.sleep(60*10)   # every 10 minutes
            else:
                return



import os

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
OUTPUT_DIR = os.path.join(BASE_DIR, 'generated')
MAX_SOUND_LIFE = 60*60*12   # seconds of sound file storing

INSTALLED_APPS = [
    'rest_framework',
    'tts',
]


# coding: utf-8
import os
import json
import shutil

from rest_framework.test import APITestCase
from rest_framework import status
from django.core.urlresolvers import reverse
from django.conf import settings


class TestGenerateView(APITestCase):
    url = reverse('speach_synthesizer:generate')

    def tearDown(self):
        shutil.rmtree(settings.OUTPUT_DIR)

    def test_generate(self):
        response = self.client.post(self.url, {'text': 'Awesome Text!'})
        content = json.loads(response.content.decode('utf-8'))

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(content['status'], 0)
        self.assertTrue(os.path.exists('%s/%s.wav' % (settings.OUTPUT_DIR, content['uuid'])))


class TestGetFileView(APITestCase):
    get_url = reverse('speach_synthesizer:get_file')
    gen_url = reverse('speach_synthesizer:generate')

    def test_get_file(self):
        gen_response = self.client.post(self.gen_url, {'text': 'Awesome Text!'})
        gen_content = json.loads(gen_response.content.decode('utf-8'))

        get_response = self.client.get(self.get_url, {'uuid': gen_content['uuid']})

        self.assertEqual(get_response.status_code, status.HTTP_200_OK)
        self.assertEqual(get_response['Content-Disposition'], 'attachment; filename=%s.wav' % gen_content['uuid'])



# coding: utf-8
from rest_framework import serializers
from . import models


class Generation(serializers.Serializer):
    text = serializers.CharField(required=True)


class GetFile(serializers.Serializer):
    uuid = serializers.UUIDField(required=True)

    def validate_uuid(self, value):
        if not models.SoundFile.objects.filter(uuid=value).exists():
            raise serializers.ValidationError('Sound file with uuid "%s" not found' % value)
        return value
